import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.el.ELException;
import javax.el.Expression;
import javax.el.ValueExpression;
import javax.faces.FacesException;
import javax.faces.context.FacesContext;

import org.ajax4jsf.model.DataVisitor;
import org.ajax4jsf.model.ExtendedDataModel;
import org.ajax4jsf.model.Range;
import org.ajax4jsf.model.SequenceRange;
import org.richfaces.component.SortOrder;
import org.richfaces.model.Arrangeable;
import org.richfaces.model.ArrangeableState;
import org.richfaces.model.FilterField;
import org.richfaces.model.SortField;

public abstract class LazyDataModel<E> extends ExtendedDataModel<Object> implements Arrangeable {

	private static final long serialVersionUID = 1L;
	private Integer dataItem;
	private SequenceRange cachedRange;
	private List<E> cachedItems;
	private List<FilterField> filterFields;
	private List<SortField> sortFields;
	private int rowCount;
	
	@Override
	public Object getRowKey() {
		return dataItem;
	}

	@Override
	public void setRowKey(Object dataItem) {
		this.dataItem = (Integer)dataItem;
	}

	@Override
	public void walk(FacesContext facesContext, DataVisitor visitor, Range range, Object argument) {
		 SequenceRange sequenceRange = (SequenceRange) range;
         if ((this.cachedItems == null || !areEqualRanges(this.cachedRange, sequenceRange)) && sequenceRange != null) {
             int first = sequenceRange.getFirstRow();
             int rows = sequenceRange.getRows();
             if(!(rows > 0)){
            	 rows = 10;
             }
             Map<String, Boolean> sorts = getSorts(facesContext);
             Map<String, String> filters = getFilters(facesContext);
             String sortField = null;
             boolean sortOrder = false;
             this.cachedRange = sequenceRange;
             this.cachedItems = load(first, rows, sortField, sortOrder, filters);
         }
         for (int i=0; i<cachedItems.size();i++) {
             visitor.process(facesContext, i, argument);
         }
	}

	@Override
	public int getRowCount() {
		return rowCount;
	}

	public void setRowCount(int rowCount) {
		this.rowCount = rowCount;
	}
	
	@Override
	public Object getRowData() {
		return cachedItems.get(dataItem);
	}

	@Override
	public int getRowIndex() {
		return -1;
	}

	@Override
	public Object getWrappedData() {
		return null;
	}

	@Override
	public boolean isRowAvailable() {
		return (this.dataItem != null);
	}

	@Override
	public void setRowIndex(int arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	public void setWrappedData(Object arg0) {
		// TODO Auto-generated method stub

	}
	public void arrange(FacesContext context, ArrangeableState state) {
		if(state!=null) {
			this.filterFields = state.getFilterFields();
			this.sortFields = state.getSortFields();
		}
        this.cachedItems = null;
        this.cachedRange = null;
	}
	
	private static boolean areEqualRanges(SequenceRange range1,
			SequenceRange range2) {
		if (range1 == null || range2 == null) {
			return range1 == null && range2 == null;
		} else {
			return range1.getFirstRow() == range2.getFirstRow()
					&& range1.getRows() == range2.getRows();
		}
	}

	private Map<String, String> getFilters(FacesContext facesContext) {
		Map<String, String> filters = null;
		if (filterFields != null) {
			filters = new HashMap<String, String>();
			for (FilterField filterField : filterFields) {
				String propertyName = getPropertyName(facesContext,
						filterField.getFilterExpression());
				String filterValue = filterField.getFilterValue().toString();
				if (filterValue != null && filterValue.length() != 0) {
					filters.put(propertyName, filterValue);
				}
			}
		}
		return filters;
	}

	private Map<String, Boolean> getSorts(FacesContext facesContext) {
		Map<String, Boolean> sorts = null;
		if (sortFields != null) {
			sorts = new HashMap<String, Boolean>();
			for (SortField sortField : sortFields) {
				SortOrder ordering = sortField.getSortOrder();
				String propertyName = getPropertyName(facesContext,sortField.getSortBy());
				boolean ascending = SortOrder.ascending.equals(ordering);
				sorts.put(propertyName, ascending);
			}
		}
		return sorts;
	}

	private String getPropertyName(FacesContext facesContext,
			Expression expression) {
		try {
			return (String) ((ValueExpression) expression)
					.getValue(facesContext.getELContext());
		} catch (ELException e) {
			throw new FacesException(e.getMessage(), e);
		}
	}

	public abstract List<E> load(int first, int pageSize, String sortField,
			boolean sortOrder, Map<String, String> filters);
}